@@ -26,6 +26,9 @@
 */
 
 #include "sdlapp.h"
+#include "display.h"
+#include "logger.h"
+#include "SDL_syswm.h"
 
 std::string gSDLAppResourceDir;
 std::string gSDLAppConfDir;
@@ -40,49 +43,56 @@ std::string gSDLAppTitle = "SDL App";
 std::string gSDLAppExec  = "sdlapp";
 
 #ifdef _WIN32
-HWND gSDLAppConsoleWindow = 0;
 
-void SDLAppCreateWindowsConsole() {
-    if(gSDLAppConsoleWindow !=0) return;
+HWND SDLApp::console_window   = 0;
+bool SDLApp::existing_console = false;
 
-    //create a console on Windows so users can see messages
+void SDLApp::showConsole(bool show) {
+    if(!SDLApp::console_window) return;
+
+    ShowWindow( SDLApp::console_window, show);
+}
+
+void SDLApp::initConsole() {
+    if(SDLApp::console_window != 0) return;
+
+    SDLApp::console_window = GetConsoleWindow();
+
+    // check if this is a new console or not
+    CONSOLE_SCREEN_BUFFER_INFO buffer_info;
+    if(GetConsoleScreenBufferInfo( GetStdHandle( STD_OUTPUT_HANDLE ), &buffer_info )) {
+        existing_console = !(buffer_info.dwCursorPosition.X==0 && buffer_info.dwCursorPosition.Y==0);
+    // assume TERM env implies this is an existing terminal
+    } else if(getenv("TERM") != 0) {
+        existing_console = true;
+    }
+
+    // don't customize the console unless it was created for this program
+    if(existing_console) return;
+
+    //disable the close button so the user cant crash the application
+    HMENU hm = GetSystemMenu(SDLApp::console_window, false);
+    DeleteMenu(hm, SC_CLOSE, MF_BYCOMMAND);
+
+    //set title
 
-    //find an available name for our window
-    int console_suffix = 0;
     char consoleTitle[512];
+    int console_suffix = 0;
     sprintf(consoleTitle, "%s Console", gSDLAppTitle.c_str());
 
     while(FindWindow(0, consoleTitle)) {
         sprintf(consoleTitle, "%s Console %d", gSDLAppTitle.c_str(), ++console_suffix);
     }
 
-    AllocConsole();
     SetConsoleTitle(consoleTitle);
-
-    //redirect streams to console
-    freopen("conin$", "r", stdin);
-    freopen("conout$","w", stdout);
-    freopen("conout$","w", stderr);
-
-    gSDLAppConsoleWindow = 0;
-
-    //wait for our console window
-    while(gSDLAppConsoleWindow==0) {
-        gSDLAppConsoleWindow = FindWindow(0, consoleTitle);
-        SDL_Delay(100);
-    }
-
-    //disable the close button so the user cant crash gource
-    HMENU hm = GetSystemMenu(gSDLAppConsoleWindow, false);
-    DeleteMenu(hm, SC_CLOSE, MF_BYCOMMAND);
 }
 
-void SDLAppResizeWindowsConsole(int height) {
-    if(gSDLAppConsoleWindow !=0) {
+void SDLApp::resizeConsole(int height) {
+    if(SDLApp::console_window !=0 && !existing_console) {
         RECT windowRect;
-        if(GetWindowRect(gSDLAppConsoleWindow, &windowRect)) {
+        if(GetWindowRect(SDLApp::console_window, &windowRect)) {
             float width = windowRect.right - windowRect.left;
-            MoveWindow(gSDLAppConsoleWindow,windowRect.left,windowRect.top,width,height,true);
+            MoveWindow(SDLApp::console_window,windowRect.left,windowRect.top,width,height,true);
         }
     }
 }
@@ -107,14 +117,16 @@ std::string SDLAppAddSlash(std::string path) {
 //info message
 void SDLAppInfo(std::string msg) {
 #ifdef _WIN32
-    SDLAppCreateWindowsConsole();
+    SDLApp::showConsole(true);
 #endif
 
     printf("%s\n", msg.c_str());
 
 #ifdef _WIN32
-    printf("\nPress Enter\n");
-    getchar();
+    if(!SDLApp::existing_console) {
+        printf("\nPress Enter\n");
+        getchar();
+    }
 #endif
 
     exit(0);
@@ -125,20 +137,45 @@ void SDLAppQuit(std::string error) {
     SDL_Quit();
 
 #ifdef _WIN32
-    SDLAppCreateWindowsConsole();
+    SDLApp::showConsole(true);
 #endif
 
     fprintf(stderr, "%s: %s\n", gSDLAppExec.c_str(), error.c_str());
     fprintf(stderr, "Try '%s --help' for more information.\n\n", gSDLAppExec.c_str());
 
 #ifdef _WIN32
-    fprintf(stderr, "Press Enter\n");
-    getchar();
+    if(!SDLApp::existing_console) {
+        fprintf(stderr, "Press Enter\n");
+        getchar();
+    }
 #endif
 
     exit(1);
 }
 
+bool SDLApp::getClipboardText(std::string& text) {
+
+#if SDL_VERSION_ATLEAST(2,0,0)
+    char* clipboard_text = SDL_GetClipboardText();
+
+    if(clipboard_text!=0) {
+        text = std::string(clipboard_text);
+    } else {
+        text.resize(0);
+    }
+
+    return true;
+#else
+    return false;
+#endif
+}
+
+void SDLApp::setClipboardText(const std::string& text) {
+#if SDL_VERSION_ATLEAST(2,0,0)
+    SDL_SetClipboardText(text.c_str());
+#endif
+}
+
 void SDLAppInit(std::string apptitle, std::string execname) {
     gSDLAppTitle = apptitle;
     gSDLAppExec  = execname;
@@ -146,7 +183,9 @@ void SDLAppInit(std::string apptitle, std::string execname) {
     std::string conf_dir     = "";
     std::string resource_dir = "data/";
     std::string fonts_dir    = "data/fonts/";
+    std::string shaders_dir  = "data/shaders/";
 #ifdef _WIN32
+
     char szAppPath[MAX_PATH];
     GetModuleFileName(0, szAppPath, MAX_PATH);
 
@@ -159,6 +198,8 @@ void SDLAppInit(std::string apptitle, std::string execname) {
     conf_dir     = path + std::string("\\");
     resource_dir = path + std::string("\\data\\");
     fonts_dir    = path + std::string("\\data\\fonts\\");
+    shaders_dir  = path + std::string("\\data\\shaders\\");
+
 #else
     //get working directory
     char cwd_buff[1024];
@@ -167,6 +208,7 @@ void SDLAppInit(std::string apptitle, std::string execname) {
         conf_dir     = std::string(cwd_buff) + std::string("/");
         resource_dir = std::string(cwd_buff) + std::string("/") + resource_dir;
         fonts_dir    = std::string(cwd_buff) + std::string("/") + fonts_dir;
+        shaders_dir  = std::string(cwd_buff) + std::string("/") + shaders_dir;
     }
 
 #endif
@@ -181,6 +223,7 @@ void SDLAppInit(std::string apptitle, std::string execname) {
     if (SDLAppDirExists(SDLAPP_RESOURCE_DIR)) {
         resource_dir = SDLAPP_RESOURCE_DIR;
         fonts_dir    = SDLAPP_RESOURCE_DIR + std::string("/fonts/");
+        shaders_dir  = SDLAPP_RESOURCE_DIR + std::string("/shaders/");
     }
 #endif
 
@@ -190,21 +233,26 @@ void SDLAppInit(std::string apptitle, std::string execname) {
     }
 #endif
 
-    resource_dir = SDLAppAddSlash(resource_dir);
-    conf_dir     = SDLAppAddSlash(conf_dir);
-    fonts_dir    = SDLAppAddSlash(fonts_dir);
+    resource_dir  = SDLAppAddSlash(resource_dir);
+    conf_dir      = SDLAppAddSlash(conf_dir);
+    fonts_dir     = SDLAppAddSlash(fonts_dir);
+    shaders_dir   = SDLAppAddSlash(shaders_dir);
 
     texturemanager.setDir(resource_dir);
     fontmanager.setDir(fonts_dir);
+    shadermanager.setDir(shaders_dir);
 
     gSDLAppResourceDir = resource_dir;
     gSDLAppConfDir     = conf_dir;
+    gSDLAppShaderDir   = shaders_dir;
+
+    fontmanager.init();
 }
 
 void SDLAppParseArgs(int argc, char *argv[], int* xres, int* yres, bool* fullscreen, std::vector<std::string>* otherargs) {
 
     for (int i=1; i<argc; i++) {
-        debugLog("argv[%d] = %s\n", i, argv[i]);
+        debugLog("argv[%d] = %s", i, argv[i]);
 
         std::string args(argv[i]);
 
@@ -236,7 +284,7 @@ void SDLAppParseArgs(int argc, char *argv[], int* xres, int* yres, bool* fullscr
                 int height = atoi(heightstr.c_str());
 
                 if(width>0 && height>0) {
-                    debugLog("w=%d, h=%d\n",width,height);
+                    debugLog("w=%d, h=%d",width,height);
 
                     *xres = width;
                     *yres = height;
@@ -256,6 +304,7 @@ SDLApp::SDLApp() {
     fps=0;
     return_code=0;
     appFinished=false;
+    min_delta_msec = 8;
 }
 
 void SDLApp::updateFramerate() {
@@ -281,6 +330,58 @@ void SDLApp::stop(int return_code) {
     appFinished=true;
 }
 
+bool SDLApp::handleEvent(SDL_Event& event) {
+
+    switch(event.type) {
+        case SDL_QUIT:
+            quit();
+            break;
+
+        case SDL_MOUSEMOTION:
+            mouseMove(&event.motion);
+            break;
+
+#if SDL_VERSION_ATLEAST(2,0,0)
+        case SDL_TEXTINPUT:
+            textInput(&event.text);
+             break;
+
+        case SDL_TEXTEDITING:
+            textEdit(&event.edit);
+            break;
+
+        case SDL_MOUSEWHEEL:
+            mouseWheel(&event.wheel);
+            break;
+
+        case SDL_WINDOWEVENT:
+            if(event.window.event == SDL_WINDOWEVENT_RESIZED) {
+                resize(event.window.data1, event.window.data2);
+            }
+            break;
+#else
+        case SDL_VIDEORESIZE:
+            resize(event.resize.w, event.resize.h);
+            break;
+#endif
+
+        case SDL_MOUSEBUTTONDOWN:
+        case SDL_MOUSEBUTTONUP:
+            mouseClick(&event.button);
+            break;
+
+        case SDL_KEYDOWN:
+        case SDL_KEYUP:
+            keyPress(&event.key);
+            break;
+
+        default:
+            return false;
+    }
+
+    return true;
+}
+
 int SDLApp::run() {
 
     Uint32 msec=0, last_msec=0, buffer_msec=0, total_msec = 0;
@@ -293,15 +394,20 @@ int SDLApp::run() {
     msec = SDL_GetTicks();
     last_msec = msec;
 
+#if SDL_VERSION_ATLEAST(2,0,0)
+        //text input seems to be enabled by default, turn it off
+        SDL_StopTextInput();
+#endif
+
     while(!appFinished) {
         last_msec = msec;
         msec      = SDL_GetTicks();
 
         Uint32 delta_msec = msec - last_msec;
 
-        // cant have delta ticks be 0
+        // cant have delta ticks be less than 8ms
         buffer_msec += delta_msec;
-        if(buffer_msec < 1) {
+        if(buffer_msec < min_delta_msec) {
             SDL_Delay(1);
             continue;
         }
@@ -325,35 +431,7 @@ int SDLApp::run() {
         //process new events
         SDL_Event event;
         while ( SDL_PollEvent(&event) ) {
-
-            switch(event.type) {
-                case SDL_QUIT:
-                    appFinished=true;
-                    break;
-
-                case SDL_MOUSEMOTION:
-                    mouseMove(&event.motion);
-                    break;
-
-                case SDL_MOUSEBUTTONDOWN:
-                    mouseClick(&event.button);
-                    break;
-
-                case SDL_MOUSEBUTTONUP:
-                    mouseClick(&event.button);
-                    break;
-
-                case SDL_KEYDOWN:
-                    keyPress(&event.key);
-                    break;
-
-                case SDL_KEYUP:
-                    keyPress(&event.key);
-                    break;
-
-                default:
-                    break;
-            }
+            handleEvent(event);
         }
 
         update(t, dt);
