@@ -28,17 +28,16 @@
 #ifndef STRINGHASH_H
 #define STRINGHASH_H
 
-#include <stdlib.h>
 #include <string>
 
 #include "vectors.h"
 
 //basic string hash algorithm
-int   intHash(int key);
-int   stringHash(const std::string& str);
-vec2f vec2Hash(const std::string& str);
-vec3f vec3Hash(const std::string& str);
-vec3f colourHash(const std::string& str);
-vec3f colourHash2(const std::string& str);
+int stringHash(const std::string& str);
+vec2 vec2Hash(const std::string& str);
+vec3 vec3Hash(const std::string& str);
+vec3 colourHash(const std::string& str);
+
+extern int gStringHashSeed;
 
 #endif
