@@ -21,10 +21,17 @@
 #include "core/vectors.h"
 
 #include <string>
+#include <vector>
+#include <map>
+#include <time.h>
 
 class LogEntry {
 
 private:
+    static std::vector<std::string> fields;
+    static std::vector<std::string> default_fields;
+    static std::map<std::string, std::string> field_titles;
+
     std::string maskHostname(const std::string& hostname);
 public:
     LogEntry();
@@ -33,6 +40,10 @@ public:
     void setSuccess();
     void setResponseColour();
 
+    bool getValue(const std::string& field, std::string& value) const;
+
+    std::string log_entry;
+
     time_t timestamp;
 
     std::string hostname;
@@ -41,6 +52,8 @@ public:
     std::string path;
 
     std::string pid;
+    std::string method;
+    std::string protocol;
 
     std::string response_code;
     long response_size;
@@ -51,6 +64,10 @@ public:
     vec3 response_colour;
 
     bool successful;
+
+    static const std::vector<std::string>& getFields();
+    static const std::vector<std::string>& getDefaultFields();
+    static const std::string& getFieldTitle(const std::string& field);
 };
 
 class AccessLog {
