@@ -74,6 +74,10 @@ float Paddle::getX() {
     return pos.x;
 }
 
+void Paddle::setX(float x) {
+    pos.x = x;
+}
+
 RequestBall* Paddle::getTarget() {
     return target;
 }
@@ -86,16 +90,16 @@ void Paddle::setTarget(RequestBall* target) {
         return;
     }
 
-    vec2 dest = target->finish();
+    vec2 dest = target->getFinishPos();
     vec4 col  = (settings.paddle_mode == PADDLE_VHOST || settings.paddle_mode == PADDLE_PID)  ?
-        vec4(token_colour,1.0) : vec4(target->colour, 1.0f);
+        vec4(token_colour,1.0) : vec4(target->getColour(), 1.0f);
 
     moveTo((int)dest.y, target->arrivalTime(), col);
 }
 
 bool Paddle::mouseOver(TextArea& textarea, vec2& mouse) {
 
-    if(pos.x <= mouse.x && pos.x + width >= mouse.x && abs(pos.y - mouse.y) < height/2) {
+    if(!token.empty() && pos.x <= mouse.x && pos.x + width >= mouse.x && abs(pos.y - mouse.y) < height/2) {
 
         std::vector<std::string> content;
 
